/*

EGYPT Toolkit for Statistical Machine Translation
Written by Yaser Al-Onaizan, Jan Curin, Michael Jahr, Kevin Knight, John Lafferty, Dan Melamed, David Purdy, Franz Och, Noah Smith, and David Yarowsky.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
USA.

*/
/* Perplexity.cc
 * =============
 * Mike Jahr, 7/21/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 * 
 * Last Modified by: Yaser Al-Onaizan, August 17, 1999
 *
 * Simple class used to calculate cross entropy and perplexity
 * of models.
 */

#include "Perplexity.h"

Array2<double, Vector<double> > *Perplexity::E_M_L = 0 ; // initialize E(M/L).
const int Perplexity::initialized = Perplexity::Initialize();


void Perplexity::record(string model){
  modelid.push_back(model);
  perp.push_back(perplexity());
  ce.push_back(cross_entropy());
}
// static table initialization function
int Perplexity::Initialize()
{
  E_M_L  = new Array2<double, Vector<double> >(MAX_SENTENCE_LENGTH,MAX_SENTENCE_LENGTH);
  
  // Fill the table for calculating the scaling factor for cross entropy
  unsigned int l, m ;

  Vector<double> fact(MAX_SENTENCE_LENGTH, 1.0);
  for (m = 2 ; m < MAX_SENTENCE_LENGTH ; m++)
    fact[m] = fact[m-1] * m ;
  for (m = 1 ; m < MAX_SENTENCE_LENGTH ; m++)
    for (l = 1 ; l < MAX_SENTENCE_LENGTH ; l++) {
      (*E_M_L)(l, m) = log (pow((LAMBDA * l), m) * exp(-LAMBDA * l) / 
			   (fact[m])) ;
    }

  return(true);
}
